module PuzzleReader
  def self.read(puzzle_file)
    File.open puzzle_file do |f|
      @test_cases = DataSet.new(f.lines)
    end
  end

  class DataSet
    attr_reader :data

    def self
      @data
    end

    def initialize(data)
      @data ||= []
      begin 
        current_row = data.next.strip!
        peek_row    = data.peek.strip
        return if current_row.nil?
      end while current_row.empty?

      if current_row =~ /^\d*$/
        rows = current_row.to_i.times
        if peek_row.empty?
          rows.each { @data << DataSet.new(data) } 
        else
          rows.each { @data << DataRow.new(data) }
        end
      end
    end

    def each(&block)
      data.each(&block)
    end

    def method_missing(m, *args, &block)
      begin
        @data.send(m, *args, &block)
      rescue NoMethodError
        super
      end
    end
  end

  class DataRow < DataSet

    def initialize(data)
      @data = data.next.to_s.strip
    end
  end
end
